@@ -95,7 +95,8 @@ run (void *cls,
     return;
   }
   if (NULL ==
-      (plugin = CHALLENGER_DB_plugin_load (cfg)))
+      (plugin = CHALLENGER_DB_plugin_load (cfg,
+                                           false)))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to initialize database plugin.\n");
@@ -182,6 +183,30 @@ run (void *cls,
     enum GNUNET_DB_QueryStatus qs;
     uint64_t row_id;
 
+    qs = plugin->client_check2 (plugin->cls,
+                                redirect_uri,
+                                client_secret,
+                                &row_id);
+    switch (qs)
+    {
+    case GNUNET_DB_STATUS_SOFT_ERROR:
+    case GNUNET_DB_STATUS_HARD_ERROR:
+      GNUNET_break (0);
+      global_ret = EXIT_FAILURE;
+      goto cleanup;
+    case GNUNET_DB_STATUS_SUCCESS_NO_RESULTS:
+      break;
+    case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
+      if (be_quiet)
+        fprintf (stdout,
+                 "%llu\n",
+                 (unsigned long long) row_id);
+      else
+        fprintf (stdout,
+                 "Client added. Client ID is: %llu\n",
+                 (unsigned long long) row_id);
+      goto cleanup;
+    }
     qs = plugin->client_add (plugin->cls,
                              redirect_uri,
                              client_secret,
@@ -252,12 +277,8 @@ main (int argc,
   };
   enum GNUNET_GenericReturnValue ret;
 
-  /* FIRST get the libtalerutil initialization out
-     of the way. Then throw that one away, and force
-     the CHALLENGER defaults to be used! */
-  (void) TALER_project_data_default ();
-  GNUNET_OS_init (CHALLENGER_project_data_default ());
-  ret = GNUNET_PROGRAM_run (argc, argv,
+  ret = GNUNET_PROGRAM_run (CHALLENGER_project_data (),
+                            argc, argv,
                             "challenger-admin CLIENT_REDIRECT_URI",
                             "Tool to add, modify or remove clients from challenger",
                             options,
