@@ -662,7 +662,8 @@ run (void *cls,
                              &rc);
   rc = GNUNET_CURL_gnunet_rc_create (CH_ctx);
   if (NULL ==
-      (CH_db = CHALLENGER_DB_plugin_load (config)))
+      (CH_db = CHALLENGER_DB_plugin_load (config,
+                                          false)))
   {
     global_ret = EXIT_NOTINSTALLED;
     GNUNET_SCHEDULER_shutdown ();
@@ -733,12 +734,8 @@ main (int argc,
   };
   enum GNUNET_GenericReturnValue ret;
 
-  /* FIRST get the libtalerutil initialization out
-     of the way. Then throw that one away, and force
-     the CHALLENGER defaults to be used! */
-  (void) TALER_project_data_default ();
-  GNUNET_OS_init (CHALLENGER_project_data_default ());
-  ret = GNUNET_PROGRAM_run (argc, argv,
+  ret = GNUNET_PROGRAM_run (CHALLENGER_project_data (),
+                            argc, argv,
                             "challenger-httpd",
                             "challenger REST and OAuth 2.0 API",
                             options,
