@@ -246,13 +246,6 @@ reply_error (struct ChallengeContext *bc,
 {
   struct CH_HandlerContext *hc = bc->hc;
 
-  if (0 == CH_get_output_type (hc->connection))
-    return TALER_TEMPLATING_reply_error (
-      hc->connection,
-      template,
-      http_status,
-      ec,
-      hint);
   return TALER_MHD_reply_with_error (
     hc->connection,
     http_status,
@@ -744,7 +737,6 @@ CH_handler_challenge (struct CH_HandlerContext *hc,
 
   {
     json_t *args;
-    enum GNUNET_GenericReturnValue ret;
     struct MHD_Response *resp;
     unsigned int http_status;
     MHD_RESULT res;
@@ -767,30 +759,10 @@ CH_handler_challenge (struct CH_HandlerContext *hc,
                                       CH_validation_duration)))
       );
     http_status = MHD_HTTP_OK;
-    if (0 == CH_get_output_type (hc->connection))
-    {
-      ret = TALER_TEMPLATING_build (hc->connection,
-                                    &http_status,
-                                    "enter-tan-form",
-                                    NULL,
-                                    NULL,
-                                    args,
-                                    &resp);
-      json_decref (args);
-      if (GNUNET_SYSERR == ret)
-      {
-        GNUNET_break (0);
-        return MHD_NO;
-      }
-      GNUNET_break (GNUNET_OK == ret);
-    }
-    else
-    {
-      GNUNET_break (0 ==
-                    json_object_del (args,
-                                     "nonce"));
-      resp = TALER_MHD_make_json (args);
-    }
+    GNUNET_break (0 ==
+                  json_object_del (args,
+                                   "nonce"));
+    resp = TALER_MHD_make_json (args);
     GNUNET_break (MHD_YES ==
                   MHD_add_response_header (resp,
                                            MHD_HTTP_HEADER_CACHE_CONTROL,
