@@ -1,6 +1,6 @@
 /*
   This file is part of Challenger
-  Copyright (C) 2023 Taler Systems SA
+  Copyright (C) 2023, 2024 Taler Systems SA
 
   Challenger is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -24,15 +24,89 @@
 #include "challenger-httpd_setup.h"
 #include "challenger-httpd_common.h"
 
+struct SetupContext
+{
+
+  /**
+   * Our handler context.
+   */
+  struct CH_HandlerContext *hc;
+
+  /**
+   * Opaque parsing context.
+   */
+  void *opaque_post_parsing_context;
+
+  /**
+   * Uploaded JSON data, NULL if upload is not yet complete.
+   */
+  json_t *root;
+
+};
+
+
+/**
+ * Callback to clean up the request context.
+ *
+ * @param[in,out] ctx a `struct SetupContext` to clean up
+ */
+static void
+request_done (void *ctx)
+{
+  struct SetupContext *sc = ctx;
+
+  if (NULL == ctx)
+    return;
+  if (NULL != sc->root)
+  {
+    json_decref (sc->root);
+    sc->root = NULL;
+  }
+  TALER_MHD_parse_post_cleanup_callback (sc->opaque_post_parsing_context);
+  GNUNET_free (sc);
+}
+
 
 MHD_RESULT
 CH_handler_setup (struct CH_HandlerContext *hc,
                   const char *upload_data,
                   size_t *upload_data_size)
 {
+  struct SetupContext *sc = hc->ctx;
   unsigned long long client_id;
   const char *client_secret;
 
+  if (NULL == sc)
+  {
+    /* Fresh request, do initial setup */
+    sc = GNUNET_new (struct SetupContext);
+    sc->hc = hc;
+    hc->cc = &request_done;
+    hc->ctx = sc;
+  }
+  if ( (NULL == sc->root) &&
+       (0 != *upload_data_size) )
+  {
+    /* parse byte stream upload into JSON */
+    enum GNUNET_GenericReturnValue res;
+
+    res = TALER_MHD_parse_post_json (hc->connection,
+                                     &sc->opaque_post_parsing_context,
+                                     upload_data,
+                                     upload_data_size,
+                                     &sc->root);
+    if (GNUNET_SYSERR == res)
+    {
+      GNUNET_assert (NULL == sc->root);
+      return MHD_NO; /* bad upload, could not even generate error */
+    }
+    if ( (GNUNET_NO == res) ||
+         (NULL == sc->root) )
+    {
+      GNUNET_assert (NULL == sc->root);
+      return MHD_YES; /* so far incomplete upload or parser error */
+    }
+  }
   (void) upload_data;
   (void) upload_data_size;
   {
@@ -101,7 +175,8 @@ CH_handler_setup (struct CH_HandlerContext *hc,
     qs = CH_db->setup_nonce (CH_db->cls,
                              client_id,
                              &nonce,
-                             expiration_time);
+                             expiration_time,
+                             NULL);
     switch (qs)
     {
     case GNUNET_DB_STATUS_HARD_ERROR:
