@@ -25,6 +25,7 @@
 #include "challenger-httpd_common.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_signatures.h>
+#include "challenger_cm_enums.h"
 
 
 /**
@@ -67,6 +68,11 @@ struct TokenContext
    * Uploaded 'grant_type' field from POST data.
    */
   char *grant_type;
+
+  /**
+   * Uploaded 'code_verifier' field from POST data.
+   */
+  char *code_verifier;
 };
 
 
@@ -90,6 +96,7 @@ cleanup_ctx (void *cls)
   GNUNET_free (bc->client_secret);
   GNUNET_free (bc->code);
   GNUNET_free (bc->grant_type);
+  GNUNET_free (bc->code_verifier);
   GNUNET_free (bc);
 }
 
@@ -149,6 +156,10 @@ post_iter (void *cls,
       .name = "grant_type",
       .ptr = &bc->grant_type
     },
+    {
+      .name = "code_verifier",
+      .ptr = &bc->code_verifier
+    },
     {
       .name = NULL,
       .ptr = NULL
@@ -356,16 +367,21 @@ CH_handler_token (struct CH_HandlerContext *hc,
     char *client_scope;
     char *client_state;
     char *client_redirect_uri;
+    char *code_challenge;
+    uint32_t code_challenge_method;
     enum GNUNET_DB_QueryStatus qs;
     char *code;
+    enum CHALLENGER_CM code_challenge_method_enum;
 
-    qs = CH_db->validation_get (CH_db->cls,
-                                &bc->nonce,
-                                &client_secret,
-                                &address,
-                                &client_scope,
-                                &client_state,
-                                &client_redirect_uri);
+    qs = CH_db->validation_get_pkce (CH_db->cls,
+                                     &bc->nonce,
+                                     &client_secret,
+                                     &address,
+                                     &client_scope,
+                                     &client_state,
+                                     &client_redirect_uri,
+                                     &code_challenge,
+                                     &code_challenge_method);
     switch (qs)
     {
     case GNUNET_DB_STATUS_HARD_ERROR:
@@ -374,7 +390,7 @@ CH_handler_token (struct CH_HandlerContext *hc,
         hc->connection,
         MHD_HTTP_INTERNAL_SERVER_ERROR,
         TALER_EC_GENERIC_DB_FETCH_FAILED,
-        "validation_get");
+        "validation_get_pkce");
     case GNUNET_DB_STATUS_SOFT_ERROR:
       GNUNET_break (0);
       return MHD_NO;
@@ -385,10 +401,158 @@ CH_handler_token (struct CH_HandlerContext *hc,
         MHD_HTTP_UNAUTHORIZED,
         "invalid_grant",
         TALER_EC_CHALLENGER_GENERIC_VALIDATION_UNKNOWN,
-        "validation_get");
+        "validation_get_pkce");
     case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
       break;
     }
+
+    code_challenge_method_enum = CHALLENGER_cm_from_int (
+      code_challenge_method);
+    if (CHALLENGER_CM_UNKNOWN == code_challenge_method_enum)
+    {
+      GNUNET_break (0);
+      return TALER_MHD_reply_with_error (
+        hc->connection,
+        MHD_HTTP_INTERNAL_SERVER_ERROR,
+        TALER_EC_GENERIC_PARAMETER_MALFORMED,
+        "Invalid code_challenge_method");
+    }
+
+    /* Verify the code_challenge if present*/
+    if (NULL != code_challenge)
+    {
+      if (NULL == bc->code_verifier)
+      {
+        GNUNET_break_op (0);
+        GNUNET_free (client_scope);
+        GNUNET_free (client_secret);
+        GNUNET_free (client_redirect_uri);
+        GNUNET_free (client_state);
+        GNUNET_free (code_challenge);
+        return TALER_MHD_reply_with_oauth_error (
+          hc->connection,
+          MHD_HTTP_UNAUTHORIZED,
+          "invalid_grant",
+          TALER_EC_GENERIC_PARAMETER_MISSING,
+          "code_verifier is missing");
+      }
+
+      switch (code_challenge_method_enum)
+      {
+      case CHALLENGER_CM_S256:
+        {
+          gcry_md_hd_t hd;
+          unsigned char hash[32];
+          char *encoded_hash = NULL;
+          size_t encoded_len;
+          const void *md;
+
+          if (GPG_ERR_NO_ERROR !=
+              gcry_md_open (&hd,
+                            GCRY_MD_SHA256,
+                            0))
+          {
+            GNUNET_break (0);
+            GNUNET_free (client_scope);
+            GNUNET_free (client_secret);
+            GNUNET_free (client_redirect_uri);
+            GNUNET_free (client_state);
+            GNUNET_free (code_challenge);
+            return TALER_MHD_reply_with_oauth_error (
+              hc->connection,
+              MHD_HTTP_INTERNAL_SERVER_ERROR,
+              "server_error",
+              TALER_EC_GENERIC_INTERNAL_INVARIANT_FAILURE,
+              "Failed to initialize SHA256 hash function");
+          }
+          gcry_md_write (hd,
+                         bc->code_verifier,
+                         strlen (bc->code_verifier));
+          md = gcry_md_read (hd,
+                             0);
+          GNUNET_assert (NULL != md);
+          memcpy (hash,
+                  md,
+                  sizeof (hash));
+          gcry_md_close (hd);
+
+          encoded_len
+            = GNUNET_STRINGS_base64url_encode (hash,
+                                               sizeof (hash),
+                                               &encoded_hash);
+
+          if ( (0 == encoded_len) ||
+               (NULL == encoded_hash) )
+          {
+            GNUNET_break (0);
+            GNUNET_free (client_scope);
+            GNUNET_free (client_secret);
+            GNUNET_free (client_redirect_uri);
+            GNUNET_free (client_state);
+            GNUNET_free (code_challenge);
+            return TALER_MHD_reply_with_oauth_error (
+              hc->connection,
+              MHD_HTTP_INTERNAL_SERVER_ERROR,
+              "server_error",
+              TALER_EC_GENERIC_INTERNAL_INVARIANT_FAILURE,
+              "Failed to encode hash to Base64 URL");
+          }
+
+          if (0 != strcmp (encoded_hash,
+                           code_challenge))
+          {
+            GNUNET_break_op (0);
+            GNUNET_free (client_scope);
+            GNUNET_free (client_secret);
+            GNUNET_free (client_redirect_uri);
+            GNUNET_free (client_state);
+            GNUNET_free (code_challenge);
+            return TALER_MHD_reply_with_oauth_error (
+              hc->connection,
+              MHD_HTTP_UNAUTHORIZED,
+              "invalid_grant",
+              TALER_EC_CHALLENGER_CLIENT_FORBIDDEN_BAD_CODE,
+              "code_verifier does not match code_challenge (SHA256)");
+          }
+        }
+        break;
+      case CHALLENGER_CM_PLAIN:
+        {
+          if (0 != strcmp (bc->code_verifier,
+                           code_challenge))
+          {
+            GNUNET_break_op (0);
+            GNUNET_free (client_scope);
+            GNUNET_free (client_secret);
+            GNUNET_free (client_redirect_uri);
+            GNUNET_free (client_state);
+            GNUNET_free (code_challenge);
+            return TALER_MHD_reply_with_oauth_error (
+              hc->connection,
+              MHD_HTTP_UNAUTHORIZED,
+              "invalid_grant",
+              TALER_EC_CHALLENGER_CLIENT_FORBIDDEN_BAD_CODE,
+              "code_verifier does not match code_challenge (PLAIN)");
+          }
+        }
+        break;
+      case CHALLENGER_CM_UNKNOWN:
+      case CHALLENGER_CM_EMPTY:
+        GNUNET_break (0);
+        GNUNET_free (client_scope);
+        GNUNET_free (client_secret);
+        GNUNET_free (client_redirect_uri);
+        GNUNET_free (client_state);
+        GNUNET_free (code_challenge);
+        return TALER_MHD_reply_with_oauth_error (
+          hc->connection,
+          MHD_HTTP_INTERNAL_SERVER_ERROR,
+          "server_error",
+          TALER_EC_GENERIC_DB_INVARIANT_FAILURE,
+          "Database has empty or unknown challenge mode but with code_challenge");
+      }
+    }
+
     if (NULL == address)
     {
       GNUNET_break_op (0);
@@ -396,6 +560,7 @@ CH_handler_token (struct CH_HandlerContext *hc,
       GNUNET_free (client_secret);
       GNUNET_free (client_redirect_uri);
       GNUNET_free (client_state);
+      GNUNET_free (code_challenge);
       return TALER_MHD_reply_with_oauth_error (
         hc->connection,
         MHD_HTTP_CONFLICT,
