@@ -1,7 +1,7 @@
 #!/bin/sh
 # This file is in the public domain.
 set -eu
-. telesign-secrets
+# . telesign-secrets
 # Set AUTH_TOKEN=...
 NUMBER=$(echo "$1" | jq -r .phone)
 MESSAGE=$(cat -)
