@@ -17,8 +17,14 @@
 -- Everything in one big transaction
 BEGIN;
 
--- Unregister patch (0001.sql)
-SELECT _v.unregister_patch('challenger-0001');
+WITH xpatches AS (
+  SELECT patch_name
+  FROM _v.patches
+  WHERE starts_with(patch_name,'challenger-')
+)
+  SELECT _v.unregister_patch(xpatches.patch_name)
+  FROM xpatches;
+
 DROP SCHEMA challenger CASCADE;
 
 -- And we're out of here...
